library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use work.all;

entity counter_tb is
end counter_tb;

architecture behavior of counter_tb is
  constant cycle_time_c : time:=1 ns;
  constant counter_width_c : natural:=4;

  -- clock_gen
  signal clk_s : std_logic;      
  signal reset_s : std_logic;

  -- counter
  signal count_s : std_logic_vector(counter_width_c-1 downto 0);
  signal count_event_s : std_logic;
  signal enable_s : std_logic;

begin
  
  clock_gen : entity work.clock_gen(behavioral)
    generic map (
      TCYC => cycle_time_c)
    port map (
      clk   => clk_s,
      reset => reset_s );
  
 counter : entity work.counter(behavioral)
    generic map (
      WIDTH => counter_width_c )
    port map (
      CLK => clk_s,
      CLR => reset_s,
      EN => enable_s,
      CE => count_event_s,
      Q => count_s );
  
  testbench : process

    type pattern_type is record
      en, ce  : std_logic;
    end record;

    type pattern_array is array (natural range <>) of pattern_type;
    constant patterns : pattern_array :=
      (('0', '0'),
       ('0', '0'),
       ('1', '0'),
       ('1', '1'),
       ('1', '1'),
       ('1', '0'),
       ('1', '1'),
       ('1', '0'),
       ('1', '1'),
       ('1', '0'),
       ('0', '1'),
       ('1', '1'),
       ('1', '0'),
       ('0', '0'));
    
  begin
    
    for i in patterns'range loop
      enable_s <= patterns(i).en;
      count_event_s <= patterns(i).ce;
      wait for cycle_time_c;
    end loop;
    wait;
    
  end process testbench;
  
end behavior;
